#include "platform.h"

// Windows (Win32) platform wrapper functions


// sleep (pause thread execution for 'msec' milliseconds)
// less milliseconds than timeslice of system is not meaningful (e.g. Windows ~10ms)
VOID sleep_milliseconds( UINT32 msec )
{
	Sleep( msec );
}

// remap winsock error code to a more general (useful on any platform) error code
INT32 getlasterror()
{
	INT32 lasterr = WSAGetLastError();
	INT32 retcode;

	switch( lasterr )
	{
		case WSAEADDRNOTAVAIL:	retcode = EADDRNOTAVAIL;
								break;

		case WSAEAFNOSUPPORT:	retcode = EAFNOSUPPORT;
								break;

		case WSAEALREADY:		retcode = EALREADY;
								break;

		case WSAEBADF:			retcode = EBADF;
								break;

		case WSAECONNREFUSED:	retcode = ECONNREFUSED;
								break;

		case WSAEINPROGRESS:	retcode = EINPROGRESS;
								break;

		case WSAEINTR:			retcode = EINTR;
								break;

		case WSAEISCONN:		retcode = EISCONN;
								break;

		case WSAENETUNREACH:	retcode = ENETUNREACH;
								break;

		case WSAENOTSOCK:		retcode = ENOTSOCK;
								break;

		case WSAEPROTOTYPE:		retcode = EPROTOTYPE;
								break;

		case WSAETIMEDOUT:		retcode = ETIMEDOUT;
								break;

		case WSAEADDRINUSE:		retcode = EADDRINUSE;
								break;

		case WSAECONNRESET:		retcode = ECONNRESET;
								break;

		case WSAEHOSTUNREACH:	retcode = EHOSTUNREACH;
								break;

		case WSAEINVAL:			retcode = EINVAL;
								break;

		case WSAEHOSTDOWN:		retcode = EHOSTDOWN;
								break;

		case WSAENAMETOOLONG:	retcode = ENAMETOOLONG;
								break;

		case WSAENETDOWN:		retcode = ENETDOWN;
								break;

		case WSAEWOULDBLOCK:	retcode = EAGAIN;
								break;

		default:				retcode = EUNKNOWN;
								break;

	}

	return retcode;
}

// create a socket for use with TCP or UDP wrapper
socktype create_inet_socket( INT32 type )
{
	return( socket( AF_INET, type, 0 ) );
}

// connect a previously created socket to the remote endpoint defined via Saddr 
INT32 connect_socket(socktype s, const struct sockaddr * Saddr, size_t size_sockaddr)
{	
	return( connect( s, Saddr, size_sockaddr) );
}

// close or cleanup a socket (connected, but must work on unconnected sockets, too)
INT32	close_socket( socktype s )
{
	shutdown(s, SD_SEND);

	return( closesocket( s ) );
}

// wrapper for receive data on a connected socket
INT32 receive_socket_function( socktype s, UCHAR *buf, UINT32 buflen)
{
	return( recv(s, (CHAR *) buf, (INT32) buflen, 0) );
}

//
// wrapper in order to put the socket to nonblocking mode 
// (so that the inner loop of grabframes() works properly)
//
// 0 == success
// or SOCKET_ERROR on error
//
INT32 set_socket_nonblocking ( socktype s )
{
	unsigned long nonblock = 1;
	
	return( ioctlsocket( s, FIONBIO, &nonblock) );
}