#ifndef __VIDEOFILE_LOAD_SW__
#define __VIDEOFILE_LOAD_SW__

////////////////////////////////////////////////////////////////////////////////////
//
// videofileload.h  
//
//  header file required for loading in video files created by the PITZ video system
//
////////////////////////////////////////////////////////////////////////////////////
//	created and modified by Stefan Weisse / DESY Zeuthen, November 2005
////////////////////////////////////////////////////////////////////////////////////

#include "videosystem.h"

// type of file, also used as type information for data stored in a VIDINFHDR

#define		FCN_NONE			0
#define		FCN_AVERAGE			1
#define		FCN_ENVELOPE		2 
#define		FCN_SIGNIFICANCE	3 


#define		TYPE_NONE_EMPTY			-1
#define		TYPE_CHOOSE			 0
#define		TYPE_IMM			 1
#define		TYPE_IMC			 2 
#define		TYPE_BKG			 3
#define		TYPE_BKC			 4
#define		TYPE_SIGNIFICANCE	 5

// required structure

struct VIDINFHDR
{
	UINT32	width;
	UINT32	height;
	UINT32	bpp;
	UINT32	bpp_effective;
	
	DOUBLE	scale;
	
	INT32	numframes;
	
	INT32	type; // FILE_

	VOID  **framebits;
	DOUBLE *framescales;

};

// structures needed for loading in compressed files (*.imc, *.bkc)

typedef struct 
{
  UINT32 width;
  UINT32 height;
  UINT32 bpp; // may contain bpp_effective (!)
  UINT32 num_frames;
} FHDR;

typedef struct {
  DOUBLE scale;
  UINT32 len_compressed;
  UINT32 len_uncompressed;
} IMHDR;


// function prototypes

VS_EXPORT void	initialize_vidinfhdr		( struct VIDINFHDR *vinf );
VS_EXPORT void	free_vidinfhdr			( struct VIDINFHDR *vinf );

VS_EXPORT INT32	load_imm_file			( CHAR *filename, struct VIDINFHDR *vinf );
VS_EXPORT INT32	load_bkg_file			( CHAR *filename, struct VIDINFHDR *vinf );
VS_EXPORT INT32	load_imc_bkc_file		( CHAR *filename, struct VIDINFHDR *vinf, INT32 type);
VS_EXPORT INT32	load_imc_file			( CHAR *filename, struct VIDINFHDR *vinf );
VS_EXPORT INT32	load_bkc_file			( CHAR *filename, struct VIDINFHDR *vinf );
VS_EXPORT INT32	load_file			( INT32 type, char *filename, struct VIDINFHDR *vinf );


VS_EXPORT INT32	save_file			( INT32 type, char *filename, struct VIDINFHDR *vinf );
VS_EXPORT INT32	save_imm_file			( char *outfilename, struct VIDINFHDR *vinf );
VS_EXPORT INT32	save_bkg_file			( char *outfilename, struct VIDINFHDR *vinf );
VS_EXPORT INT32	save_bkc_file			( char *outfilename, struct VIDINFHDR *vinf );
VS_EXPORT INT32	save_imc_file			( char *outfilename, struct VIDINFHDR *vinf );




#endif  // __VIDEOFILE_LOAD_SW__
