#include <stdio.h>
#include <stdlib.h>
#include <sys/timeb.h>
#include "videosystem.h"
#include "videofileload.h"

typedef struct VideoHdrStruct
{ 
  int kennung;
  int compression;
  int len_total;
  int len_data;
  int len_fmthdr;
  int frameno;
  struct timeb timestamp;
  int cameraport;
  int widthframe;
  int heightframe;
  int compressed;
  unsigned char Reserved[12];
  double scr_scale;
  int bpp;
  int bpp_effective;
  unsigned char reserved1[8];
} VIDHDR;

int getFrameFromImageBlob(unsigned char *frame,const unsigned char *imageBlob,int *frameSize, int *frameWidth, int *frameHeight)
{
  int siz;
  VIDHDR *videoHdr = (VIDHDR *)imageBlob;
  static int sizvh = sizeof(VIDHDR);

  if (frameSize == NULL) return 20; // argument_list_error;
  memcpy(videoHdr,imageBlob,sizvh);

  if (videoHdr->kennung != 0x11223344)
  {
    *frameSize = 0;
    return 103; // invalid_data;
  }

  siz = videoHdr->heightframe * videoHdr->widthframe * videoHdr->bpp / 8;
  if (siz > *frameSize)
  {
    *frameSize = siz;
    return 23; //dimension_error;
  }
  if (videoHdr->compressed == 1)
  { // is compressed
    decompress_huffyuv(frame,(VOIDP)&imageBlob[sizvh + videoHdr->len_fmthdr],siz);
  }
  else
  { // is not compressed    
    memcpy(frame,&imageBlob[sizvh],siz);
  }
  *frameSize = siz;
  if (frameWidth != NULL) *frameWidth = videoHdr->widthframe;
  if (frameHeight != NULL) *frameHeight = videoHdr->heightframe;
  return 0;
}

